package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.BlurryUserDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskImportDto;
import com.qmth.distributed.print.business.bean.query.BlurryParam;
import com.qmth.distributed.print.business.bean.query.ExamTaskParam;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskTemp;
import com.qmth.distributed.print.business.mapper.ExamTaskMapper;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.service.ExamTaskTempService;
import com.qmth.distributed.print.common.enums.ExamStatusEnum;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskServiceImpl extends ServiceImpl<ExamTaskMapper, ExamTask> implements ExamTaskService {

    @Autowired
    private ExamTaskTempService examTaskTempService;

    @Override
    public List<ExamTask> listByCourseCode(Long schoolId, String code) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getCourseCode, code);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<ExamTaskDto> list(ExamTaskParam param) {
        Page<ExamTaskDto> page = new Page<>(param.getPageNumber(), param.getPageSize());
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listPage(page, param);
        return examTaskDtoIPage;
    }

    @Override
    public List<String> listPaperNumber(BlurryParam param) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(param.getParam())) {
            queryWrapper.lambda().eq(ExamTask::getSchoolId, param.getSchoolId())
                    .and(i -> i.like(ExamTask::getCourseCode, param.getParam()).or().like(ExamTask::getCourseName, param.getParam()));
        }
        List<ExamTask> examTasks = this.list(queryWrapper);
        if (examTasks != null && examTasks.size() > 0) {
            list = examTasks.stream().map(m -> m.getPaperNumber()).distinct().collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public List<BlurryUserDto> listUser(BlurryParam param) {
        return this.baseMapper.listUser(param);
    }

    @Override
    public boolean assignUser(ExamTask examTask) {
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId()).eq(ExamTask::getId, examTask.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean enable(ExamTask examTask) {
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getEnable, examTask.getEnable()).eq(ExamTask::getId, examTask.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean saveExamTask(ExamTask examTask) {
        // todo paperNumber生成问题， 必填字段校验
        return this.save(examTask);
    }

    @Override
    public List<ExamTaskImportDto> importFile(MultipartFile file, String md5, ApiUser apiUser) {
        // 导入文件
        return null;
    }

    @Override
    public boolean saveBatch(ExamTask task) {
        if(StringUtils.isBlank(task.getBatchNo())){
            throw ExceptionResultEnum.ERROR.exception("batchNo不能为空");
        }
        QueryWrapper<ExamTaskTemp> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskTemp::getBatchNo, task.getBatchNo());
        List<ExamTaskTemp> examTaskTemps = examTaskTempService.list(queryWrapper);
        if(examTaskTemps.isEmpty()){
            throw ExceptionResultEnum.ERROR.exception("没有可保存数据");
        }
        List<ExamTask> list = new ArrayList<>();
        for (ExamTaskTemp examTaskTemp : examTaskTemps) {
            ExamTask examTask = new ExamTask();
            examTask.setSchoolId(task.getSchoolId());
            examTask.setCourseCode(examTaskTemp.getCourseCode());
            examTask.setCourseName(examTaskTemp.getCourseName());
            examTask.setSpecialty(examTaskTemp.getSpecialty());
            examTask.setPaperNumber(examTaskTemp.getPaperNumber());
            examTask.setStartTime(task.getStartTime());
            examTask.setEndTime(task.getEndTime());
            examTask.setCardRuleId(task.getCardRuleId());
            examTask.setStatus(ExamStatusEnum.NEW);
            examTask.setBatchNo(task.getBatchNo());
            examTask.setCreateId(task.getCreateId());
            examTask.setCreateTime(System.currentTimeMillis());
            String userId = task.getUsers().stream().map(m->{
                if (examTaskTemp.getCourseCode().equals(m.get("courseCode"))) {
                    return m.get("userId");
                }
                return "";
            }).findFirst().get();
            if(StringUtils.isNotBlank(userId)){
                examTask.setUserId(Long.valueOf(userId));
            }
            list.add(examTask);
        }
        return this.saveBatch(list);
    }
}
