package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.CardRuleDto;
import com.qmth.distributed.print.business.bean.query.BlurryParam;
import com.qmth.distributed.print.business.bean.query.CardRuleParam;
import com.qmth.distributed.print.business.entity.BasicCardRule;
import com.qmth.distributed.print.business.service.BasicCardRuleService;
import com.qmth.distributed.print.business.util.RequestUtils;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 题卡规则 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "题卡规则Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.basic}/card_rule")
@Aac(auth = BOOL.FALSE)
public class BasicCardRuleController {

    @Autowired
    private BasicCardRuleService basicCardRuleService;

    /**
     * 查询
     *
     * @param request
     * @param param
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request, CardRuleParam param) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        IPage<CardRuleDto> cardRuleDtoIPage = basicCardRuleService.list(param);
        return ResultUtil.ok(cardRuleDtoIPage);
    }

    /**
     * 模糊查询
     *
     * @param request
     * @param param
     * @return
     */
    @ApiOperation(value = "模糊查询")
    @RequestMapping(value = "/query", method = RequestMethod.GET)
    public Result list(HttpServletRequest request, BlurryParam param) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        List<BasicCardRule> cardRules = basicCardRuleService.list(param);
        return ResultUtil.ok(cardRules);
    }

    /**
     * 查询单个
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "查询单个")
    @RequestMapping(value = "/get_one", method = RequestMethod.GET)
    public Result getOne(Long id) {
        BasicCardRule cardRule = basicCardRuleService.getById(id);
        return ResultUtil.ok(cardRule);
    }

    /**
     * 新增/修改
     * @param request
     * @param cardRule
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody BasicCardRule cardRule){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        cardRule.setSchoolId(apiUser.getSchoolId());
        cardRule.setCreateId(cardRule.getId() == null ? apiUser.getUserData().getId() : null);
        cardRule.setUpdateId(cardRule.getId() != null ? apiUser.getUserData().getId() : null);
        boolean isSuccess = basicCardRuleService.saveCardRule(cardRule);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     *
     * @param cardRule
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody BasicCardRule cardRule) {
        boolean isSuccess = basicCardRuleService.enable(cardRule);
        return ResultUtil.ok(isSuccess);
    }
}

