package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.UserDto;
import com.qmth.distributed.print.business.bean.query.UserParam;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.service.SysUserService;
import com.qmth.distributed.print.business.util.RequestUtils;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * <p>
 * 用户表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "用户Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/user")
@Aac(auth = BOOL.FALSE)
public class SysUserController {

    @Autowired
    private SysUserService sysUserService;

    /**
     * 查询
     * @param param
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request, UserParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        IPage<UserDto> userDtos = sysUserService.list(param);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增/修改
     * @param user
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody SysUser user){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        user.setSchoolId(apiUser.getSchoolId());
        user.setCreateId(user.getId() == null ? apiUser.getUserData().getId() : null);
        user.setUpdateId(user.getId() != null ? apiUser.getUserData().getId() : null);
        boolean isSuccess = sysUserService.saveUser(user);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     * @param user
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysUser user){
        boolean isSuccess = sysUserService.enable(user);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 重置密码
     * @param user
     * @return
     */
    @ApiOperation(value = "重置密码")
    @RequestMapping(value = "/reset_password", method = RequestMethod.POST)
    public Result resetPassword(@RequestBody SysUser user){
        boolean isSuccess = sysUserService.resetPassword(user.getId());
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 修改密码
     * @return
     */
    @ApiOperation(value = "修改密码")
    @RequestMapping(value = "/update_password", method = RequestMethod.POST)
    public Result updatePassword(@RequestBody Map<String, Object> user){
        Long id = Long.valueOf(user.get("id").toString());
        String oldPassword = user.get("oldPassword").toString();
        String password = user.get("password").toString();
        boolean isSuccess = sysUserService.updatePassword(id, oldPassword, password);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 绑定角色
     * @param sysUser
     * @return
     */
    @ApiOperation(value = "绑定角色")
    @RequestMapping(value = "/bind_roles", method = RequestMethod.POST)
    public Result bindRoles(@RequestBody SysUser sysUser){
        boolean isSuccess = sysUserService.bindRoles(sysUser);
        return ResultUtil.ok(isSuccess);
    }
}

